@@ -14,14 +14,14 @@ typedef unsigned short umode_t;
 typedef uint8_t __u8;
 typedef uint16_t __u16;
 typedef uint32_t __u32;
-#ifndef __STRICT_ANSI__
+#if !defined(__STRICT_ANSI__) || __STDC_VERSION__ + 0 >= 199900L
 typedef uint64_t __u64;
 #endif
 
 typedef int8_t __s8;
 typedef int16_t __s16;
 typedef int32_t __s32;
-#ifndef __STRICT_ANSI__
+#if !defined(__STRICT_ANSI__) || __STDC_VERSION__ + 0 >= 199900L
 typedef int64_t __s64;
 #endif
 
