@@ -7,12 +7,8 @@
 
 __BEGIN_DECLS
 
-#define gettext(msgid) dgettext(0,msgid)
-#define dgettext(domainname,msgid) dcgettext(domainname,msgid,LC_MESSAGES)
-
-#define ngettext(msgid1,msgid2,n) dngettext(0,msgid1,msgid2,n)
-#define dngettext(dn,msgid1,msgid2,n) dngettext(dn,msgid1,msgid2,n,LC_MESSAGES)
-
+char* gettext(const char* msgid) __THROW;
+char* dgettext(const char* domainname, const char* msgid) __THROW;
 char* dcgettext(const char *domainname, const char *msgid, int category) __THROW;
 char* dcngettext(const char *domainname,
 		 const char *msgid1, const char *msgid2,
@@ -22,6 +18,13 @@ char* textdomain(const char *domainname) __THROW;
 char* bindtextdomain(const char *domainname, const char *dirname) __THROW;
 char* bind_textdomain_codeset(const char *domainname, const char *codeset) __THROW;
 
+#define gettext(msgid) dgettext(0,msgid)
+#define dgettext(domainname,msgid) dcgettext(domainname,msgid,LC_MESSAGES)
+
+#define ngettext(msgid1,msgid2,n) dngettext(0,msgid1,msgid2,n)
+#define dngettext(dn,msgid1,msgid2,n) dngettext(dn,msgid1,msgid2,n,LC_MESSAGES)
+
+
 __END_DECLS
 
 #endif
