@@ -4,32 +4,36 @@
 #include <sys/cdefs.h>
 #include <sys/types.h>
 
+__BEGIN_DECLS
+
 typedef struct {
-	int     __val[2];
+  int     __val[2];
 } __kernel_fsid_t;
 
 struct statfs {
-	long f_type;
-	long f_bsize;
+  long f_type;
+  long f_bsize;
 #ifdef __mips__
-	long f_frsize;
+  long f_frsize;
 #endif
-	long f_blocks;
-	long f_bfree;
+  long f_blocks;
+  long f_bfree;
 #ifndef __mips__
-	long f_bavail;
+  long f_bavail;
 #endif
-	long f_files;
-	long f_ffree;
+  long f_files;
+  long f_ffree;
 #ifdef __mips__
-	long f_bavail;
+  long f_bavail;
 #endif
-	__kernel_fsid_t f_fsid;
-	long f_namelen;
-	long f_spare[6];
+  __kernel_fsid_t f_fsid;
+  long f_namelen;
+  long f_spare[6];
 };
 
 int statfs(const char *path, struct statfs *buf) __THROW;
 int fstatfs(int fd, struct statfs *buf) __THROW;
 
+__END_DECLS
+
 #endif
