@@ -2,5 +2,5 @@
 
 int islower(int c) {
   unsigned char x=c&0xff;
-  return (x>='a' && x<='z') || (x>=223 && x<=255 && x!=247);
+  return (x>='a' && x<='z') || (x>=223 && x!=247);
 }
