@@ -2,11 +2,11 @@
 #include <stdlib.h>
 #include <unistd.h>
 
-int fclose(FILE *stream) {
+int fclose_unlocked(FILE *stream) {
   int res;
   FILE *f,*fl;
-  fflush(stream);
-  res=close(stream->fd);
+  res=fflush_unlocked(stream);
+  res|=__libc_close(stream->fd);
   for (fl=0,f=__stdio_root; f; fl=f,f=f->next)
     if (f==stream) {
       if (fl)
@@ -20,3 +20,5 @@ int fclose(FILE *stream) {
   free(stream);
   return res;
 }
+
+int fclose(FILE *stream) __attribute__((weak,alias("fclose_unlocked")));
