@@ -1,7 +1,7 @@
 #include <string.h>
 #include <assert.h>
 
-main() {
+int main() {
   const char* test="blubber";
   assert(memcmp(test,"blubber",8)==0);
   assert(memcmp(test,"fnord",5)<0);
