@@ -1,11 +1,15 @@
 #define _XOPEN_SOURCE
 #include <time.h>
+#include <stdio.h>
 
-main() {
+int main() {
   char buf[1024];
   struct tm* t;
   time_t T=time(0);
   t=localtime(&T);
+  
   strftime(buf,sizeof(buf),"%c",t);
   printf("%s\n",strptime(buf,"%c",t));
+
+  return 0;
 }
